@@ -85,12 +85,12 @@ sub set_time_zone {
             if ( ref($min) )
             {
                 $min = $min->clone;
-                $min->set_time_zone( $tz );
+                $min->set_time_zone( 'floating' );
             }
             if ( ref($max) )
             {
                 $max = $max->clone;
-                $max->set_time_zone( $tz );
+                $max->set_time_zone( 'floating' ); 
             }
             return Set::Infinite::_recurrence->new( $min, $max );
         },
@@ -198,6 +198,11 @@ sub empty_set {
     return bless { set => Set::Infinite::_recurrence->new }, $class;
 }
 
+sub is_empty_set {
+    my $set = $_[0];
+    $set->{set}->is_null;
+}
+
 sub clone { 
     bless { 
         set => $_[0]->{set}->copy,
@@ -292,10 +297,30 @@ sub previous {
 sub current {
     my $self = shift;
 
-    my $return = $self->intersected_spans( $_[0] );
+    my $previous;
+    my $next;
+    {
+        my $min;
+        $min = $_[0]->min if UNIVERSAL::can( $_[0], 'union' );
+        $min = $_[0] if ! defined $min;
+        return undef if ! ref( $min ) && $min == INFINITY;
+        my $span = DateTime::Span->from_datetimes( end => $min );
+        my $iterator = $self->intersection( $span );
+        $previous = $iterator->previous;
+        $span = DateTime::Span->from_datetimes( start => $min );
+        $iterator = $self->intersection( $span );
+        $next = $iterator->next;
+    }
+    return $previous unless defined $next;
 
-    $return = $self->previous( $_[0] )
-        if ( ! defined $return->max );
+    my $dt1 = defined $previous
+        ? $next->union( $previous )
+        : $next;
+
+    my $return = $dt1->intersected_spans( $_[0] );
+
+    $return = $previous
+        if !defined $return->max;
 
     bless $return, 'DateTime::SpanSet'
         if defined $return;
@@ -496,11 +521,14 @@ sub span {
 sub duration { 
     my $dur; 
 
+    return DateTime::Duration->new( seconds => 0 ) 
+        if $_[0]->{set}->is_empty;
+
+    local $@;
     eval { 
         local $SIG{__DIE__};   # don't want to trap this (rt ticket 5434)
         $dur = $_[0]->{set}->size 
     };
-    $@ = undef;  # clear the eval() error message
 
     return $dur if defined $dur && ref( $dur );
     return DateTime::Infinite::Future->new -
@@ -542,9 +570,12 @@ DateTime::SpanSet - set of DateTime spans
 
 =head1 DESCRIPTION
 
-DateTime::SpanSet is a class that represents sets of datetime spans.
-An example would be a recurring meeting that occurs from 13:00-15:00
-every Friday.
+C<DateTime::SpanSet> is a class that represents sets of datetime
+spans.  An example would be a recurring meeting that occurs from
+13:00-15:00 every Friday.
+
+This is different from a C<DateTime::Set>, which is made of individual
+datetime points as opposed to ranges.
 
 =head1 METHODS
 
@@ -593,6 +624,12 @@ a span like C<[$dt, inf)>.
 
 Creates a new empty set.
 
+=item * is_empty_set
+
+Returns true is the set is empty; false otherwise.
+
+    print "nothing" if $set->is_empty_set;
+
 =item * clone
 
 This object method returns a replica of the given object.
@@ -658,11 +695,11 @@ This method is used to find the "current" span in the set,
 that intersects a given datetime or span. If no current span
 is found, then the "previous" span is returned.
 
-The return value is a C<DateTime::SpanSet>, or C<undef> if there is no matching
-span in the set.
+The return value is a C<DateTime::SpanSet>, or C<undef> if there is no
+matching span in the set.
 
-If a span parameter is given, it may happen that "current" 
-returns more than one span.
+If a span parameter is given, it may happen that "current" returns
+more than one span.
 
 See also: C<intersected_spans()> method.
 
@@ -670,14 +707,14 @@ See also: C<intersected_spans()> method.
 
   my $span = $set->closest( $dt );
 
-This method is used to find the "closest" span in the set,
-given a datetime or span. 
+This method is used to find the "closest" span in the set, given a
+datetime or span.
 
-The return value is a C<DateTime::SpanSet>, or C<undef> if the
-set is empty.
+The return value is a C<DateTime::SpanSet>, or C<undef> if the set is
+empty.
 
-If a span parameter is given, it may happen that "closest" 
-returns more than one span.
+If a span parameter is given, it may happen that "closest" returns
+more than one span.
 
 =item * as_list
 
@@ -688,15 +725,15 @@ Returns a list of C<DateTime::Span> objects.
 Just as with the C<iterator()> method, the C<as_list()> method can be
 limited by a span.
 
-Applying C<as_list()> to a large recurring spanset is a very expensive operation, 
-both in CPU time and in the memory used.
+Applying C<as_list()> to a large recurring spanset is a very expensive
+operation, both in CPU time and in the memory used.
 
-For this reason, when C<as_list()> operates on large recurrence sets, it will 
-return at most approximately 200 spans. For larger sets, and for I<infinite> 
-sets, C<as_list()> will return C<undef>.
+For this reason, when C<as_list()> operates on large recurrence sets,
+it will return at most approximately 200 spans. For larger sets, and
+for I<infinite> sets, C<as_list()> will return C<undef>.
 
-Please note that this is explicitly not an empty list, since an empty list is a 
-valid return value for empty sets!
+Please note that this is explicitly not an empty list, since an empty
+list is a valid return value for empty sets!
 
 If you I<really> need to extract spans from a large set, you can:
 
@@ -728,17 +765,16 @@ C<DateTime::SpanSet> object.
 
 =item * intersected_spans
 
-This method can accept a C<DateTime> list,
-a C<DateTime::Set>, a C<DateTime::Span>, or a C<DateTime::SpanSet>
-object as an argument.
+This method can accept a C<DateTime> list, a C<DateTime::Set>, a
+C<DateTime::Span>, or a C<DateTime::SpanSet> object as an argument.
 
     $set = $set1->intersected_spans( $set2 );
 
-The method always returns a C<DateTime::SpanSet> object,
-containing all spans that are intersected by the given set.
+The method always returns a C<DateTime::SpanSet> object, containing
+all spans that are intersected by the given set.
 
-Unlike the C<intersection> method, the spans are not modified.
-See diagram below:
+Unlike the C<intersection> method, the spans are not modified.  See
+diagram below:
 
                set1   [....]   [....]   [....]   [....]
                set2      [................]
@@ -781,8 +817,8 @@ Obviously, if the span you specify does is not restricted both at the
 start and end, then your iterator may iterate forever, depending on
 the nature of your set.  User beware!
 
-The C<next()> or C<previous()> methods will return C<undef> 
-when there are no more spans in the iterator.
+The C<next()> or C<previous()> methods will return C<undef> when there
+are no more spans in the iterator.
 
 =item * start_set
 
@@ -790,9 +826,11 @@ when there are no more spans in the iterator.
 
 These methods do the inverse of the C<from_sets> method:
 
-C<start_set> retrieves a DateTime::Set with the start datetime of each span.
+C<start_set> retrieves a DateTime::Set with the start datetime of each
+span.
 
-C<end_set> retrieves a DateTime::Set with the end datetime of each span.
+C<end_set> retrieves a DateTime::Set with the end datetime of each
+span.
 
 =item * map ( sub { ... } )
 
@@ -810,29 +848,26 @@ C<end_set> retrieves a DateTime::Set with the end datetime of each span.
 
 This method is the "set" version of Perl "map".
 
-It evaluates a subroutine for each element of
-the set (locally setting "$_" to each DateTime::Span)
-and returns the set composed of the results of
-each such evaluation.
+It evaluates a subroutine for each element of the set (locally setting
+"$_" to each DateTime::Span) and returns the set composed of the
+results of each such evaluation.
 
-Like Perl "map", each element of the set
-may produce zero, one, or more elements in the 
-returned value.
+Like Perl "map", each element of the set may produce zero, one, or
+more elements in the returned value.
 
-Unlike Perl "map", changing "$_" does not change
-the original set. This means that calling map
-in void context has no effect.
+Unlike Perl "map", changing "$_" does not change the original
+set. This means that calling map in void context has no effect.
 
-The callback subroutine may not be called immediately.
-Don't count on subroutine side-effects. For example,
-a C<print> inside the subroutine may happen later than you expect.
+The callback subroutine may not be called immediately.  Don't count on
+subroutine side-effects. For example, a C<print> inside the subroutine
+may happen later than you expect.
 
 The callback return value is expected to be within the span of the
 C<previous> and the C<next> element in the original set.
 
-For example: given the set C<[ 2001, 2010, 2015 ]>,
-the callback result for the value C<2010> is expected to be
-within the span C<[ 2001 .. 2015 ]>.
+For example: given the set C<[ 2001, 2010, 2015 ]>, the callback
+result for the value C<2010> is expected to be within the span C<[
+2001 .. 2015 ]>.
 
 =item * grep ( sub { ... } )
 
@@ -846,32 +881,31 @@ within the span C<[ 2001 .. 2015 ]>.
 
 This method is the "set" version of Perl "grep".
 
-It evaluates a subroutine for each element of
-the set (locally setting "$_" to each DateTime::Span)
-and returns the set consisting of those elements 
-for which the expression evaluated to true.
+It evaluates a subroutine for each element of the set (locally setting
+"$_" to each DateTime::Span) and returns the set consisting of those
+elements for which the expression evaluated to true.
 
-Unlike Perl "grep", changing "$_" does not change
-the original set. This means that calling grep
-in void context has no effect.
+Unlike Perl "grep", changing "$_" does not change the original
+set. This means that calling grep in void context has no effect.
 
 Changing "$_" does change the resulting set.
 
-The callback subroutine may not be called immediately.
-Don't count on subroutine side-effects. For example,
-a C<print> inside the subroutine may happen later than you expect.
+The callback subroutine may not be called immediately.  Don't count on
+subroutine side-effects. For example, a C<print> inside the subroutine
+may happen later than you expect.
 
 =item * iterate
 
 I<Internal method - use "map" or "grep" instead.>
 
-This function apply a callback subroutine to all elements of a set
-and returns the resulting set.
+This function apply a callback subroutine to all elements of a set and
+returns the resulting set.
 
-The parameter C<$_[0]> to the callback subroutine is a C<DateTime::Span>
-object.
+The parameter C<$_[0]> to the callback subroutine is a
+C<DateTime::Span> object.
 
-If the callback returns C<undef>, the datetime is removed from the set:
+If the callback returns C<undef>, the datetime is removed from the
+set:
 
     sub remove_sundays {
         $_[0] unless $_[0]->start->day_of_week == 7;
@@ -880,13 +914,13 @@ If the callback returns C<undef>, the datetime is removed from the set:
 The callback return value is expected to be within the span of the
 C<previous> and the C<next> element in the original set.
 
-For example: given the set C<[ 2001, 2010, 2015 ]>,
-the callback result for the value C<2010> is expected to be
-within the span C<[ 2001 .. 2015 ]>.
+For example: given the set C<[ 2001, 2010, 2015 ]>, the callback
+result for the value C<2010> is expected to be within the span C<[
+2001 .. 2015 ]>.
 
-The callback subroutine may not be called immediately.
-Don't count on subroutine side-effects. For example,
-a C<print> inside the subroutine may happen later than you expect.
+The callback subroutine may not be called immediately.  Don't count on
+subroutine side-effects. For example, a C<print> inside the subroutine
+may happen later than you expect.
 
 =back
 
